unit UTGraphSearch;
 {Copyright   2000-2005, Gary Darby,  www.DelphiForFun.org
 This program may be used or modified for any non-commercial purpose
 so long as this original notice remains in place.
 All other rights are reserved
 }

 {An Adjacency list Graph Object }


interface
  uses forms, classes, dialogs, sysutils, UIntlist;
 type
  tMethodCall = procedure of object;
  tVerboseCall= procedure(s:string) of object;

  TEdge= record
    FromNodeIndex, ToNodeIndex:Integer; {location of nodes in a headnodes list}
    Weight, work:integer;
  end;

  TNode= class(tObject) {A node of the graph}
    adjacents:array of TEdge; {the edges to adjacent nodes}
    nbradjacents:integer;
    index:integer; { location of this node in headnodes list}
    constructor create;
  end;

  {The adjacency graph object}
  TGraphList=class(TStringList) {list of head nodes}
  public
    Q:TList;  {list to hold search results}
    maxlen:integer;  {maximum key length}
    finalized:boolean; {true=nodes have been sorted and ready for adding edges}
    stop:boolean;   {stop flag checked during searching}
    nodesSearched, queuesize:integer; {counts kept by serch routines}
    constructor create;
    destructor destroy;
    function AddNode(key:string):boolean;
    function AddEdge(const key1,key2:string):boolean; overload;
    function AddEdge(const key1,key2:string; const newweight:integer):boolean; overload;
    function AddEdge(const key1,key2:string; const newweight,newweight2:integer):boolean; overload;
    procedure finalize;
    {depth first search}
    procedure MakePathsToDF(startkey,goal:string;
                            maxdepth:integer;
                            GoalFound:TMethodcall);
    {breadth first search}
    procedure MakePathsToBF(startkey,goal:string;
                            maxdepth:integer;
                            GoalFound:TMethodcall);
    {Dijkstra Shortest Path search}
    function Dijkstra(const FromNode, ToNode:string;  var ExtPath:TIntList):integer; overload;
    function Dijkstra(const FromNode, ToNode:string;  var ExtPath:TIntList;
                                    Verbosecall:TVerbosecall {optional}):integer; overload;
    procedure noverbose(s:string);
   end;

implementation


constructor TNode.create;
begin
  inherited create;
  setlength(adjacents,5); {we'll start with room for 5, then add 5 at a time}
  nbradjacents:=0;
  index:=0;{this serves as a link back to key info, could use key field instead}
end;



constructor TGraphList.create;
begin
  inherited;
  q:=Tlist.create;
  finalized:=false;
end;

destructor TGraphList.destroy;
var
  i:integer;
begin
  {free all nodes}
  for i:= 0 to count-1 do TNode(q[i]).free;
  q.free;
  inherited;
end;

function TGraphList.AddNode(key:string):boolean;
{add a new node}
var
  node:TNode;
begin
  node:=TNode.create;
  addobject(key,node);
  result:=true;
end;

function TGraphList.AddEdge(const key1,key2:string):boolean;
{For caompatibility - add an edge to a node with weight of 0}
begin
  result:=Addedge(key1, key2, 0);

end;

function TGraphList.AddEdge(const key1,key2:string; const newWeight:integer):boolean;
begin
  result:=Addedge(key1,key2,newweight,maxint);
end;

function TGraphList.AddEdge(const key1,key2:string; const newWeight,newweight2:integer):boolean;
{add an edge to a node}
var
  k1,k2:integer;
  j:integer;
  found:boolean;
  node1,node2:TNode;
begin
  if not finalized then
  begin
    showmessage('Nodes must be added and list finalized before edges are added');
    result:=false;
    exit;
  end;
  result:=true;
  If find(key1,k1) and find(key2,k2)
  then
  begin
    if newweight<>maxint then
    begin
      node1:=TNode(objects[k1]);
      with node1 do
      begin
        found:=false;
        for j:=0 to nbradjacents-1 do
        if adjacents[j].tonodeindex=k2 then
        begin
          found:=true;
          break;
        end;
        if not found then
        begin
          inc(nbradjacents);
          if nbradjacents>length(adjacents)
          then setlength(adjacents,length(adjacents)+5);
          adjacents[nbradjacents-1].ToNodeIndex:=K2;
          adjacents[nbradjacents-1].Weight:=NewWeight;
        end;
      end;
    end;
    if newWeight2<>maxint then
    begin
      node2:=TNode(objects[k2]);
      with node2 do
      begin
        found:=false;
        for j:=0 to nbradjacents-1 do
        if adjacents[j].ToNodeIndex=k1 then
        begin
          found:=true;
         break;
        end;
        if not found then
        begin
          inc(nbradjacents);
          if nbradjacents>length(adjacents)
          then setlength(adjacents,length(adjacents)+5);
          adjacents[nbradjacents-1].ToNodeIndex:=K1;
          adjacents[nbradjacents-1].Weight:=NewWeight2;
        end;
      end;
    end;
  end;
end;


procedure TGraphlist.finalize;
{sort nodes and fill in index values}
var
  i:integer;
begin
  sort;
  for i:= 0 to count-1 do
    TNode(objects[i]).index:=i;
 finalized:=true;
end;


procedure TGraphList.MakePathsToDF(startkey,goal:string;
                                   maxdepth:integer;
                                   GoalFound:TMethodCall);
{depth first search for goal}
var
  visited:array of boolean;
  nodenbr:integer;

 procedure dfs(v:integer); {recursive depth first search}
 var
   temp:TNode;
   i:integer;
 begin
   if stop then exit;
   inc(nodesSearched);
   temp:=TNode(objects[v]);  {get the node}
   Q.add(temp);              {add it to the queue}
   visited[temp.index]:=true;
   if (strings[v]=goal)  then
   begin
     queuesize:=q.count; {stats for goalfound routine}
     GoalFound;
   end;
   if (Q.Count<maxdepth) then  {if not at max depth}
   with temp do      {iterate through adjacents recursively}
     for i:= 0 to nbradjacents-1 do
     if (not visited[adjacents[i].ToNodeIndex]) then dfs(adjacents[i].ToNodeIndex);
   q.delete(Q.count-1);    {delete temp}
   visited[temp.index]:=false; {unvisit the node}
   if nodessearched mod 1024=0 then application.processmessages;
 end;

begin {MakePathsToDF}
  if not finalized then exit;
  setlength(visited,0);   {reset 'visited' array}
  setlength(visited,count);
  Q.clear;
  find(startkey,nodenbr);
  stop:=false;
  nodessearched:=0;
  dfs(nodenbr);  {start recursive  search}
end;

Type
  {object to use for breadth first searches that saves the
   node and the path that got us here}
  TBreadthObj = class(TObject)
    Path:array of integer;
    PathLength:integer;
    Node:TNode;
  end;


procedure TGraphList.MakePathsToBF(startkey,goal:string;
                        maxdepth:integer;
                        GoalFound:TMethodCall);
{breadth first search for goal}
var
  i,j:integer;
  temp, temp2:TBreadthObj;
  node2:TNode;
  nodenbr:integer;
  LocalQ:TList;
  inpath:boolean;
begin
  if not finalized then exit;
  LocalQ:=TList.create;
  try
    {make the initial breadth object}
    temp:=TBreadthObj.create;
    find(startkey,nodenbr);
    if nodenbr<0 then exit;
    temp.node:=TNode(objects[nodenbr]);
    temp.pathlength:=1;
    setlength(temp.path,maxdepth+1);
    temp.path[0]:=temp.node.index;
    LocalQ.add(temp);  {put it in the queue}
    stop:=false;    {in case user presses stop button}
    nodessearched:=0;
    While (not stop) and (LocalQ.count>0) do
    begin
      inc(nodessearched);
      temp:=Localq.items[0];{get the oldest beadthobj}
      Localq.delete(0);
      If temp.pathlength>=maxdepth {maxdepth reached, we're done}
      then
      begin
        if localQ.count>0 then
        for i:= 0 to localq.count-1 do TBreadthObj(Localq[i]).free;
        Localq.clear; {stop when we get to maxdepth}
      end
      else
      with temp.node do
      begin {go through adjacents and make new breadth objects for them}
        for i:= 0 to nbradjacents-1 do
        begin
          inpath:=false; {make sure this adjacent is not already in the path}
          for j:=0 to temp.pathlength-1 do
          if temp.path[j]=adjacents[i].ToNodeIndex then
          begin
            inpath:=true;
            break;
          end;
          if  not inpath then {not inpath, so we can add it}
          begin
            temp2:=TBreadthObj.create;
            with temp2 do
            begin
              node:=TNode(objects[temp.node.adjacents[i].ToNodeIndex]);
              path:=copy(temp.path,0,temp.pathlength);
              setlength(path,maxdepth);
              pathlength:=temp.pathlength+1;
              if pathlength>length(path)
               then
               begin
                 setlength(path,length(path)+1);
                 showmessage('Path depth extended - shouldn''t get here');
               end;
              path[pathlength-1]:=temp.node.adjacents[i].ToNodeIndex;
            end;

            {was this a goal node?}
            if strings[temp2.Node.index]=goal then
            begin
              q.clear;
              for j:=0 to temp2.pathlength-1 do
              begin
                node2:=TNode(objects[temp2.path[j]]);
                q.add(node2);
              end;
              queuesize:=localq.count;
              GoalFound;
              temp2.free;
            end
            else LocalQ.add(temp2);;
          end;
        end;
        temp.free;
      end;
      If nodesSearched mod 1024 = 0 then application.processmessages;
    end;
    finally
      if localQ.count>0 then
      for i:= 0 to localq.count-1 do TBreadthObj(Localq[i]).free;
      localQ.free;
    end;
end;

procedure TGraphlist.NoVerbose(s:string);
begin
end;

function TGraphList.Dijkstra(const FromNode, ToNode:string; var ExtPath:TIntList):integer;
begin
  result:=Dijkstra( FromNode, ToNode, Extpath, noverbose);
end;


function TGraphList.Dijkstra(const FromNode, ToNode:string; var ExtPath:TIntList;
                     Verbosecall:TVerbosecall):integer;
{find shortest path (based on sum of edge weights) from FromNode to ToNode}
(*
Dijkstra's algorithm: pseudocode
for all vertices v,
dist(v) = infinity;
dist(first) = 0;
place all vertices in set toBeChecked;
while toBeChecked is not empty
  {in this version, also stop when shortest path to a specific destination is found}
  select v: min(dist(v)) in toBeChecked;
  remove v from toBeChecked;
  for u in toBeChecked, and path from v to u exists
  {i.e. for unchecked adjacents to v}
  do
    if dist(u) > dist(v) + weight({u,v}),
    then
       dist(u) = dist(v) + weight({u,v});
       set predecessor of u to v
       save minimum distance to u in array "d"
     endif
  enddo
endwhile
*)

var
  dist,toBechecked:TIntlist;
  v,dv,u,du:integer;
  i,node,m,ix:integer;
  fromNodeIndex, ToNodeIndex:integer;
  path,d:array of integer;
  verbose:TVerboseCall;


    function Minx:integer;
    var
      i,mind:integer;
    {return the minimal distance still in ToBeChecked list}
    {The values are index pointer to the nodes, the objects are minimal distances
     from source to that node - these are all initialized with high values, so
     only nodes whoise minimal distance from source has been determined will qualify}
    begin

      result:=0;
      mind:=integer(dist.objects[0]);
      with dist do
      for i:=1 to count-1 do
      if integer(objects[i])<mind then
      begin
        result:=i;
        mind:=integer(objects[i]);
      end;
    end;



begin
  result:=-1;
  verbose:=verbosecall;
  if not finalized then exit;  {-1 'Not finalized'}
  if (not find(fromNode, FromNodeIndex)) or (not find(toNode,ToNodeIndex)) then
  begin
    result:=-2;   {-2 : 'FROM or TO node does not exist'}
    exit;
  end;
  dist:=TIntList.create;
  setlength(path, count); setlength(d,count);
  for i:=0 to count-1 do
  begin
    dist.addobject(i,TObject(maxint));
    path[i]:=i;
  end;
  dist.objects[fromNodeindex]:=tobject(0);
  {Search until all reachable nodes have been processed or the "To" node is reached}
  while (dist.count>=0) do
  begin
    m:=minx; {node index of smallest unchecked distance}
    v:=dist[m]; {node index # of min dist}
    dv:=integer(dist.objects[m]); {min dist}
    dist.delete(m);
    verbosecall('1. "Closest" unchecked is  node ' + strings[v]
                                              + ' with weight to it = '+inttostr(dv));
    if v=tonodeindex then
    begin
      verbosecall('5.  Destination node '+ strings[v]+' found!');
      break; {path copmplete, we can stop}
    end;
    with TNode(objects[v]) do
    for i:= 0 to nbradjacents-1 do
    begin
      du:=dv+adjacents[i].weight; {new distance= minimum to adjacent plus its weight}
      u:=adjacents[i].tonodeIndex; {the node to which this applies}
      verbosecall('2. Node '+strings[u]+' through node '
                                      +strings[v]+' has path weight ' +inttostr(du));
      if (dist.find(u,ix)) and (du<integer(dist.objects[ix]))
      then  {this is a new shortest distance to this node}
      begin  {so...}
        dist.objects[ix]:=TObject(du); {update the distance list with the new distance}
        path[u]:=v;  {set the path to this node to the node that got us here}
        d[u]:=du; {put the distance in the distance array}
       verbosecall('3. *** It is the smallest path so far from start to '+strings[u]);
      end;
    end;
    verbose('4. All paths from '+ strings[v]+' checked');

  end;
  if v=ToNodeIndex then
  begin
    result:=v;
    extpath.clear;
    i:=tonodeindex;

    while (path[i]<>i)  do
    begin
      extpath.addobject(i,TObject(d[i]));
      if path[i]=fromnodeindex then break;
      i:=path[i];
    end;
     extpath.addobject(fromnodeindex,TObject(0));
  end
  else result:=-3; {err -3 ='No path found'}

end;



end.
